#include <iostream.h>
#include <fstream.h>

const int MAXDIM = 10;

struct matrice {
    int righe, colonne;
    int elementi[MAXDIM][MAXDIM];
};

void carica (matrice &scacchiera, string nomeFile, bool &esito);
void stampaScacchiera (matrice scacchiera);
void schermoLeggiRipetizioni(int &ripetizioni);
void schermoNomeFile(string &nomeFile);
void simula (matrice &scacchiera, int ripetizioni);
void generaNuovoCiclo (matrice &scacchiera);
int nuovoValore (matrice scacchiera, int x, int y);
int contaVicini (matrice scacchiera, int x, int y);
void copiaScacchiera (matrice scacchiera, matrice &copia);

void main() {
    matrice scacchiera;
    bool esito = false;
    int ripetizioni;
    string nomeFile;
    schermoNomeFile(nomeFile);
    carica(scacchiera, nomeFile, esito);
    if (esito) {
        stampaScacchiera(scacchiera);
        schermoLeggiRipetizioni(ripetizioni);
        simula(scacchiera, ripetizioni);
    } else {
        cout << "Errore nel file specificato" << endl;
    }
}

void schermoNomeFile (string &nomeFile) {
    cout << "----------------------------- \n";
    cout << "--    IL GIOCO DELLA VITA  -- \n";
    cout << "----------------------------- \n";
    cout << "\n";
    cout << "Immetti il nome del file che contiene la scacchiera \n";
    cout << "Nome file -> ";
    cin >> nomeFile;
    return;
}

void carica (matrice &scacchiera, string nomeFile, bool &esito) {
    string tmp;
    int righe, colonne;
    ifstream file (nomeFile.c_str());
    file >> tmp;
    file >> righe;
    file >> tmp;
    file >> colonne;
    if (righe <= MAXDIM && colonne <= MAXDIM) {
        scacchiera.righe = righe;
        scacchiera.colonne = colonne;
        for (int i = 0; i < scacchiera.righe; i++) {
            for (int j = 0; j < scacchiera.colonne; j++) {
                file >> scacchiera.elementi[i][j];
            }
        }
        esito = true;
    } else {
        esito = false;
    }
    return;
}

void stampaScacchiera (matrice scacchiera) {
    int i, j;
    for (j = 0; j < scacchiera.colonne * 3 + 2; j++) {
        cout << "-";
    }
    cout << endl;
    for (i = 0; i < scacchiera.righe; i++) {
        cout << "|";
        for (j = 0; j < scacchiera.colonne; j++) {
            if (scacchiera.elementi[i][j] != 0) {
                cout << " * ";
            } else {
                cout << "   ";
            }
        }
        cout << "|\n";
    }
    for (j = 0; j < scacchiera.colonne * 3 + 2; j++) {
        cout << "-";
    }
    cout << endl;

}

void schermoLeggiRipetizioni (int &ripetizioni) {
    cout << "\n";
    cout << "Immetti il numero di cicli da simulare -> ";
    cin >> ripetizioni;
    while (ripetizioni < 0) {
        cout << "** Errore. Immetti il valore corretto -> ";
        cin >> ripetizioni;
    }
    cin.ignore();
    return;
}

void simula (matrice &scacchiera, int ripetizioni) {
    string tmp;
    for (int i = 0; i < ripetizioni; i++) {
        generaNuovoCiclo(scacchiera);
        cout << "\n";
        cout << "Ciclo n. " << i + 1 << endl;
        stampaScacchiera(scacchiera);
        cout << "Invio per continuare \n";
        getline(cin, tmp);
    }
    return;
}

void generaNuovoCiclo (matrice &scacchiera) {
    matrice nuovaScacchiera;
    nuovaScacchiera.righe = scacchiera.righe;
    nuovaScacchiera.colonne = scacchiera.colonne;
    for (int i = 0; i < scacchiera.righe; i++) {
        for (int j = 0; j < scacchiera.colonne; j++) {
            nuovaScacchiera.elementi[i][j] = nuovoValore(scacchiera, i, j);
        }
    }
    copiaScacchiera(nuovaScacchiera, scacchiera);
    return;
}

int nuovoValore(matrice scacchiera, int x, int y) {
    int vecchio = scacchiera.elementi[x][y];
    int nuovo = vecchio;
    int vicini = contaVicini (scacchiera, x, y);
    if (vecchio == 1) {
        if (vicini < 2 || vicini > 3) {
            nuovo = 0;
        }
    } else {
        if (vicini == 3) {
            nuovo = 1;
        }
    }
    return nuovo;
}

int contaVicini (matrice scacchiera, int x, int y) {
    int conta = 0;
    for (int i = x - 1; i <= x + 1; i++) {
        for (int j = y - 1; j <= y + 1; j++) {
            if (i >= 0 && i < scacchiera.righe && j >= 0 && j < scacchiera.colonne) {
                conta += scacchiera.elementi[i][j];
            }
        }
    }
    return conta - scacchiera.elementi[x][y];
}

void copiaScacchiera(matrice scacchiera, matrice &copia) {
    copia.righe = scacchiera.righe;
    copia.colonne = scacchiera.colonne;
    for (int i = 0; i < scacchiera.righe; i++) {
        for (int j = 0; j < scacchiera.colonne; j++) {
            copia.elementi[i][j] = scacchiera.elementi[i][j];
        }
    }
    return;
}