#include <iostream.h>
#include <fstream.h>
#include "listaPtr2.h"

// SOTTOPROGRAMMI DI SERVIZIO:
//---------------------------

void aggiungiInTesta(lista &l, tipoValori elem, bool &esito);
elemento* puntatoreAllUltimo(lista l);
elemento* puntatoreAllaPos (lista l, int pos);
bool posizioneEsistente (lista l, int pos);

//---------------------------

void inizializza (lista &l) {
    l.first = NULL;
    l.numeroElementi = 0;
    return;
}

void leggi (lista &l) {
    cout << "\nLettura dei valori della lista" << endl;
    cout << "------------------------------" << endl;
    cout << endl;
    cout << "Quanti valori contiene la lista ? ";
    int numero;
    cin >> numero;
    while (numero < 0) {
        cout << "ERRORE: VALORE NON AMMESSO" << endl;
        cout << "Il valore deve essere maggiore o uguale di 0 ";
        cout << "Immetti valore corretto: ";
        cin >> numero;
    }
    inizializza(l);
    for (int i = 0; i < numero; i++) {
        bool esito;
        tipoValori valore;
        cout << "--Immetti il valore n." << i << ": ";
        cin >> valore;
        aggiungi(l, valore, esito);
    }
    return;
}

void stampa (lista l) {
    cout << "\nStampa dei valori della lista" << endl;
    cout << "-----------------------------" << endl;
    if (listaVuota(l)) {
        cout << "Lista vuota" << endl;
    } else {
        int i = 0;
        elemento* p = l.first;
        while (p != NULL) {
            cout << "--Valore n." <<  i  << ": " << p->valore << endl;
            p = p->next;
            i++;
        }
    }
    return;
}

void salvaSuDisco (lista l, string nomeFile) {
    int i;
    ofstream outFile(nomeFile.c_str());
    elemento* p = l.first;
    while (p != NULL) {
        outFile << p->valore << endl;
        p = p->next;
    }
    outFile.close();
    return;
}

void carica (lista &l, string nomeFile){
    tipoValori valore;
    ifstream inFile(nomeFile.c_str());
    inizializza(l);
    while (inFile >> valore) {
        bool esito;
        aggiungi(l, valore, esito);
    }
    inFile.close();
    return;
}


void aggiungi (lista &l, tipoValori elem, bool& esito){
    if (listaVuota(l)) {
        aggiungiInTesta(l, elem, esito);
    } else {
        elemento* p = puntatoreAllUltimo(l);
        p->next = new elemento;
        if (p->next == NULL) {
            esito = false;
        } else {
            p->next->valore = elem;
            p->next->next = NULL;
            l.numeroElementi++;
            esito = true;
        }
    }
    return;
}

void aggiungi (lista &l, float elem, int pos, bool& esito) {
    if (!posizioneEsistente(l, pos)) {
        if (pos != numeroElementi(l)) {
            esito = false;
        } else {
            aggiungi(l, elem, esito);
        }
    } else {
        if (pos == 0) {
            aggiungiInTesta(l, elem, esito);
        } else {
            elemento* p = puntatoreAllaPos (l, pos - 1);
            elemento* tmp = (p->next);
            p->next = new elemento;
            if (p->next == NULL) {
                esito=false;
                p->next = tmp;
            } else {
                p->next->valore = elem;
                p->next->next = tmp;
                l.numeroElementi++;
                esito = true;
            }
        }
    }
    return;
}

void elimina (lista &l, bool& esito){
    if (listaVuota(l)) {
        esito = false;
    } else {
        elemento* tmp;
        if (l.first->next == NULL) {
            tmp = l.first;
            l.first = NULL;
        } else {
            elemento* p = puntatoreAllaPos(l, numeroElementi(l) - 2);
            tmp = p->next;
            p->next = NULL;
        }
        delete tmp;
        l.numeroElementi--;
        esito = true;
    }
    return;
}

void elimina (lista &l, int pos, bool& esito){
    if (!posizioneEsistente(l, pos)) {
        esito = false;
    } else {
        elemento* tmp;
        if (pos == 0) {
            tmp = l.first;
            l.first = l.first->next;
        } else {
            elemento* p = puntatoreAllaPos (l, pos - 1);
            tmp = p->next;
            p->next = p->next->next;
        }
        delete tmp;
        l.numeroElementi--;
        esito = true;
    }
    return;
}

int cercaPosizione (lista l, tipoValori elem) {
    elemento* tmp = l.first;
    int i =  0;
    while (tmp != NULL) {
        if (tmp->valore == elem) {
            return i;
        } else {
            tmp = tmp->next;
            i++;
        }
    }
    return -1;
}

int numeroElementi (lista l){
    return l.numeroElementi;
}

tipoValori elementoInPosizione (lista l, int pos) {
    tipoValori valore;
    elemento* tmp = puntatoreAllaPos (l, pos);
    return tmp->valore;
}

bool listaVuota (lista l) {
    return (l.first == NULL);
}

bool listaPiena (lista l) {
    return (l.first != l.first);
}


// sottoprogrammi privati

bool posizioneEsistente (lista l, int pos){
    bool esiste;
    if (listaVuota(l)) {
        esiste = false;
    } else {
        esiste = (pos >= 0 && pos < numeroElementi(l));
    }
    return esiste;
}

elemento* puntatoreAllUltimo (lista l) {
    elemento* p = l.first;
    if (p != NULL) {
        while (p->next != NULL) {
            p = p->next;
        }
    }
    return p;
}

elemento* puntatoreAllaPos (lista l, int pos) {
    elemento* p = l.first;
    if (p != NULL) {
        for (int i = 0; i < pos; i++) {
            p = p->next;
        }
    }
    return p;
}

void aggiungiInTesta (lista &l, tipoValori elem, bool& esito){
    elemento* tmp = l.first;
    l.first = new elemento;
    if (l.first == NULL) {
        esito = false;
        l.first = tmp;
    } else {
        l.first->valore = elem;
        l.first->next = tmp;
        esito = true;
        l.numeroElementi++;
    }
    return;
}
